/* heatexchanger.c */
/* heatexchanger.dynamic.c chriwi */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include "global.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define HEATEX_MAX 15
#define AutoSetting -99
#define SetQ AutoSetting
#define SetT AutoSetting
#define CALC_PRESSURE_LOSS 1

struct fluid_plug {
 double Q; /* enthalphy delta_Q [Ws] */
 double m; /* delta_m [kg] */
 double cp; /* spec heat [kWs/kgK] */
 double T; /* plug temp or ambient temp [C]  */
 double U; /* U-value to ambient [W/K] */
 double h; /* timestep [s] */
};

int initPlugFluid(struct fluid_plug *Plug,double Q,double m,double cp,double T,
                  double U,double h)
{
/* init a plug with parameter values, write a struct element if AutoSetting is on
*/
if (Q ==AutoSetting) Plug->Q = m*cp*T;    else Plug->Q =Q;
if (m ==AutoSetting) Plug->m = Q/(cp*T);  else Plug->m =m;
if (cp==AutoSetting) Plug->cp= Q/(m*T);   else Plug->cp=cp;
if (T ==AutoSetting) { /* write if mdot>0 */
 if (m>0)            Plug->T = Q/(m*cp);} else Plug->T =T;
Plug->U=U; /* no autosetting */
Plug->h=h; /* no autosetting */

return (1);
}; /* initPlugFluid() */

int Get_MixedPlug_Fluid(struct fluid_plug *PlugOut,
                        struct fluid_plug *Node,
                        struct fluid_plug *Node_loss,
                        struct fluid_plug PlugIn)
{
/* Get_MixedPlug_Fluid is mixing PlugIn to this Node, calculating
 stabilized euler heat-transfer to ambient (Node_loss) and finally
 defines PlugOut
*/
#define PEDANTIC 0
double Tambient=(*Node_loss).T;
double kA=(*Node_loss).U;
double T_tmp; /* tmp calculation ot temperature */
double Qnode_tmp,Qloss;
double h=(*Node_loss).h; /* get timestep thru this element */
double m_cp; /* mixed plug */


/* add incoming plug */
(*Node).Q+=PlugIn.Q;
(*Node).m+=PlugIn.m;
m_cp=(*Node).m*(*Node).cp; 
(*Node).T=(*Node).Q/m_cp; /* get current temp */
/* calc losses 1st time */
Qloss=kA*( (*Node).T-Tambient)*h;
Qnode_tmp=(*Node).Q-Qloss;
T_tmp=Qnode_tmp/m_cp;
/* check euler stability of state equation */
if ( ( (Qloss>0) && (T_tmp<Tambient) ) || ( (Qloss<0) && (T_tmp>Tambient) ) ) {
 Qloss=m_cp*( (*Node).T-Tambient); /* reduce to ambient level */
 /* copy reduced U-value to Node_loss plug kA=Qloss/((T-Tamb)*h) */
 (*Node_loss).U=Qloss/(( (*Node).T-Tambient)*h) ; 
 if (PEDANTIC) printf("Get_MixedPlug_Fluid: U_set:%.1f U_stabilized:%.2f "
                      "Qloss:%.1f T:%.1f T_tmp:%.1f Tamb:%.1f\n",
                       kA,(*Node_loss).U,Qloss,(*Node).T,T_tmp,Tambient);
};
/* Q_loss is valid, finally calc of mixed plug state equation */
(*Node).Q-=Qloss;
(*Node).T=(*Node).Q/m_cp; /* get temp 2nd time */

(*Node_loss).Q=Qloss;
(*Node_loss).m=0; /* no mass flow */
/* split mixed plug to nodePlug and OutPlug */
(*PlugOut).m=PlugIn.m; /* copy from inlet */
(*PlugOut).Q=PlugIn.m* (*Node).cp *(*Node).T; /* has mixed plug temp */
(*PlugOut).cp=(*Node).cp;
(*PlugOut).T=(*Node).T;
/* finally modify NodePlug */
(*Node).m-=PlugIn.m;
(*Node).Q-=(*PlugOut).Q;
if ( (*Node).cp!=PlugIn.cp) {
 printf("Get_MixedPlug_Fluid: Node.cp:%.2f!=PlugIn.cp:%.2f\n",(*Node).cp,PlugIn.cp);
 exit (-1);
}
// printf("PlugIn.Q:%.2f Node.Q:%.2f PlugOut:%.2f mdot:%.2fkg/h Tnode:%.2f "
// "Tin:%.2f\n",
//  PlugIn.Q,(*Node).Q,(*PlugOut).Q,3600*PlugIn.m/h,(*Node).T,PlugIn.T);
return(1);
} /* Get_MixedPlug_Fluid() */

double Get_rho_fluid (int unit,double cp_fluid,double T_fluid);

double Get_Ny_fluid (int unit,double cp_fluid,double T_fluid);

double Get_Lambda_fluid (int unit,double cp_fluid,double T_fluid);

int Get_kA_heatexchanger(int unit,double tsim,int init_flag,double ratio,
                         double b_plate,double s_plate,double A_plate,
                         double Lambda_plate,double delta_channel,
                         double n_plate,double a_fit,double b_fit,double c_fit,
                         double Temp1,   double Temp2,
                         double *alpha1, double *alpha2,
                         double cp1,     double cp2,
                         double mdotIn1, double mdotIn2,
                         double *kA,
                         double *delta_pressure1, double *delta_pressure2);

int Get_Temp1_Temp2 (int unit,double tsim,int init_flag,double ratio,
                     int noHeatTransferFlag,
                     double log_cell_ratio,double cell_ratio,
                     double b_plate,double s_plate,double A_plate,
                     double Lambda_plate,double delta_channel,
                     double n_plate,double deltaT_log,
                     double Temp1Out,double Temp2Out,
                     double Temp1In, double Temp2In,
                     double alpha1,  double alpha2,
                     double kA,
                     double *Temp1,  double *Temp2);

int Get_alpha (int unit,double cp,double Temp,double mp,double A_heattrans,
               double delta_channel,double a_fit,double b_fit,double c_fit,
               double *alpha,double length_channel,
               double *delta_pressure);

int Get_delta_p (int unit,double length_channel,double delta_channel,double rho,
                 double w,double Re,double *delta_pressure);

int Get_C_heatexchanger(int unit,double n_plate,double m_exchanger,
                        double V_channel,double cp_exchanger,double Temp,
                        double cp_fluid,double *C_exchanger);

/*
 function: first heatexchanger-called--> return (nothing done)
           second time call: (next pump circle) calculation
 bugs:
 if   mp1=50& mp2=0
 then mp1=0 & mp2=50 at same time --> error energy bilance 
 kA_loss only for hot side !!, no heat transfer , if mp==0
 efficiency always 11
 Cmin,Cmax,NTU ??

001103 chriwi: bug with firsttimeflag , select_unit integrated 
020218 stebus: calc kA as a funktion of mdotIn1, mdotIn2, Temp1, Temp2
       only for water (cp=4.19)
020220 stebus: debugged new calc kA 
020325 stebus: more dynamic version: Get_kA_heatexchanger with actual mdots, 
       only temps from last timestep. Influence of massflow is much bigger than
       of temps! New temps are calced at the end of type in Get_Temp1_Temp2 
       beeing reread next timestep.
       There are no losses to ambient, yet. Tamb and kA_loss are used for 
       heatflow between 1st and 2nd side if one (or both) massflows are 0.
       Then before calcing losses, Tamb is set to Temp of neighbours node.
020408 konti: debugged calc kA for values of K0>1e13
020409 stebus: ratio==1 didnot work. Now it is checked if fabs(ratio-1)<1e12
       or not.
020502 stebus: out 13 and 14 for (total) Pexchange_1 / 2.
       calc pressure loss (hydraulic unruffled) 
*/
/*----------------------HEAT_EXCH TYPE 10 ----STATIC HEATEXCHANGER--------*/
int heatexchanger(unit,par,in_u,in_nr,out_n,out_o,deriv_o,deriv_n,h,
		  init_energy,qp_sum,tsim,init_flag,previous_unit,select_unit,
                  type_index)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int previous_unit; /* get unit which is connected with inp i */
           int    *select_unit; /* number of calls in this timestep   */
           int    *type_index; /* index of calling this type          */

{
/*  ./etc/xfig.awk         
    
   TYPE 10
   PRIORITY  6          used by config routine
   HYDRAULIC 10         4 pole fluid            

   
    PAR1: 1      []       'mode' 1=> counterflow
    PAR2: 4.19   [kJ/kgK] 'cp1' sp. heat capacity fluid 1st side
    PAR3: 4.19   [kJ/kgK] 'cp2' sp. heat capacity fluid 2nd side
    PAR4: -1     [W/K]    'kA' heat transfer coeff if -1: calc with par 8-13
    PAR5: -1     [kJ/K]   'C1' capacity hex incl fluid if -1:calc with par 14-16
    PAR6: -1     [kJ/K]   'C2' capacity hex incl fluid if -1: calc
    PAR7: 0      [W/K]    'kA_loss' heat loss coeff to ambient
    PAR8: 320    [mm]     'b_plate' width of plate
    PAR9: 0.5    [mm]     's_plate' thickness of plate
    PAR10: 0.2944 [m^2]    'A_plate' area of one plate 
    PAR11: 15    [W/mK]   'Lambda_plate' CrNi = 15, Cu = 393
    PAR12: 3     [mm]     'delta_channel' thickness of channel
    PAR13: 54    [1]      'n_plate' number of plates = 2 * n_channel + 1
    PAR14: 130   [kg]     'm_exchanger' empty weight of heatechxanger
    PAR15: 0.3   [l]      'V_channel' volume of each channel
    PAR16: 0.4   [kJ/kgK] 'cp_exchanger' CrNi = Cu = 0.4
    PAR17: 0.53  []       'a_fit' Nu=a_fit*pow(Pr,b_fit)*pow(Re,c_fit)
    PAR18: 0.4   []       'b_fit'
    PAR19: 0.63  []       'c_fit'
    PAR20: 7     [K]      'deltaT_log_max' warning if deltaT_log>deltaT_log_max
    
    INP1: 0      [Ws]     'dQin1' delta Q inlet 1st side
    INP2: 0      [kg/s]   'mdotIn1' massflow inlet 1st side
    INP3: 0      [Ws]     'dQin2' delta Q inlet 2nd side
    INP4: 0      [kg/s]   'mdotIn2' massflow inlet 2nd side
    INP5: 10     [C]      'T_amb' temp ambient
 
    OUT1:        [Ws]     'dQout1' delta Q outlet 1st side
    OUT2:        [kg/s]   'mdotOut1' massflow outlet 1st side
    OUT3:        [Ws]     'dQout2' delta Q outlet 2nd side
    OUT4:        [kg/s]   'mdotOut2' massflow outlet 2nd side
    OUT5:        [1]      'efficiency' efficiency 1st side 
    OUT6:        [Ws]     'sys_in' system inlet energy 
    OUT7:        [Ws]     'sys_out' system outlet energy 
    OUT8:        [Ws]     'Q_int' internal energy
    OUT9:        [W]      'P_total' P_echg + loss
    OUT10:       [C]      'Temp1Out' temp outlet-capacity of 1st side
    OUT11:       [C]      'Temp2Out' temp outlet-capacity of 2nd side
    OUT12:       []       'nn' --
    OUT13:       [W]      'Pexchange_1' dynamic heatgain 1st side
    OUT14:       [W]      'Pexchange_2' dynamic heatgain 2nd side
    OUT15:       [W]      'Pexchange' of static heatexchanger
    OUT16:       [W]      'Ploss' to ambient
    OUT17:       [K]      'deltaT_linear' lin mid temp difference of side 1 & 2 
    OUT18:       [1]      'noHeatTransferFlag' if mdotIn1 or dQinCold = 0
    OUT19:       [s]      't0' first execute time
    OUT20:       [C]      'Temp1In' temp inlet-capacity of 1st side
    OUT21:       [C]      'Temp2In' temp inlet-capacity of 2nd side
    OUT22:       [W/K]    'kA_eff'
    OUT23:       [K]      'deltaT_log' log funktion of In/Out temps and NTU
    OUT24:       [W/m^2K] 'alpha1' reread next timestep
    OUT25:       [W/m^2K] 'alpha2' reread next timestep
    OUT26:       [W/K]    'kA' reread next timestep
    OUT27:       [W/m^2K] 'k' heat transfer coeff
    OUT28:       [K]      'deltaT_linear' linear temp difference of side 1 & 2
    OUT29:       [W/K]    'kA_linear' Q_transfer/deltaT_linear
    OUT30:       [W/m^2K] 'k_linear' kA_linear/A
    OUT31:       [C]      'Temp1' temp for calc alpha1
    OUT32:       [C]      'Temp2' temp for calc alpha2
    OUT33:       [Pa]     'delta_pressure1' pressure loss 1st side
    OUT34:       [Pa]     'delta_pressure2' pressure loss 2nd side

    DERIV1: 10   [C]      'Temp1In' temp inlet-capacity of 1st side
    DERIV2: 10   [C]      'Temp2In' temp inlet-capacity of 2nd side
    DERIV3: 10   [C]      'Temp1Out' temp outlet-capacity of 1st side
    DERIV4: 10   [C]      'Temp2Out' temp outlet-capacity of 2nd side
*/

int    a=0; /*-----------------a=1 for analyse */
int    mode          = (int) PAR (1);
double cp1           =       PAR (2)*1e+3; /* scale to J/kgK */
double cp2           =       PAR (3)*1e+3; /* scale to J/kgK */
double kA            =       PAR (4);
double C1            =       PAR (5); /* capacity of node 1In+1Out [kJ/K] */
double C2            =       PAR (6); /* capacity of node 2In+2Out [kJ/K] */
double kA_loss       =       PAR (7)/4.0; /* heat loss to ambient for 4 nodes */
double b_plate       =       PAR (8)*1e-3; /* scale to m */
double s_plate       =       PAR (9)*1e-3; /* scale to m */
double A_plate       =       PAR (10);
double Lambda_plate  =       PAR (11);
double delta_channel =       PAR (12)*1e-3; /* scale to m */
double n_plate       =       PAR (13);
double m_exchanger   =       PAR (14);
double V_channel     =       PAR (15)*1e-3; /* scale to m^3 */
double cp_exchanger  =       PAR (16)*1e+3; /* scale to J/kgK */
double a_fit         =       PAR (17);/* fit for Nu in Get_alpha */
double b_fit         =       PAR (18);
double c_fit         =       PAR (19);
double deltaT_log_max=       PAR (20);
double dQin1         =  out_n IN (1);
double mdotIn1       =  out_n IN (2);
double dQin2         =  out_n IN (3);
double mdotIn2       =  out_n IN (4);
double T_amb         =  out_n IN (5);

double dQHexIn1,dQHexIn2;
double mdotHexIn1,mdotHexIn2;
double Temp1In,Temp2In,Temp1Out,Temp2Out;
double alpha1,alpha2;
double Temp1,Temp2;
double Q_transfer=0;
double Cdot1,Cdot2;
double ratio; /* ratio of capacity-flow (1st side): mdotIn1*cp1/(mdotIn2*cp2)*/
double NTU;   /* Number Of Transfer Units (1st side) */
double efficiency=0;/* efficiency (1st side): 
                       (Temp1In-Temp1Out)/(Temp1In-Temp2In) */
double deltaT_log=1; /* calced with NTU model*/
double cell_ratio,log_cell_ratio; /* cell_ratio (Zellenkennzahl) */
int    noHeatTransferFlag=0; 
int    mode_calc_kA=0;

double deltaT_linear; /*calced by linear function with Temp1In,2In,1Out,2Out */
double kA_linear; /* =Q_transfer/deltaT_linear */
double delta_pressure1=0; /* pressure loss 1st side */
double delta_pressure2=0; /* pressure loss 2nd side */

int    heatex_index=type_index[unit]; /* index to type counter */

static struct fluid_plug *NodeIn1  [HEATEX_MAX]; /* using type_index */
static struct fluid_plug *NodeIn2  [HEATEX_MAX];
static struct fluid_plug *NodeOut1 [HEATEX_MAX];
static struct fluid_plug *NodeOut2 [HEATEX_MAX];

struct fluid_plug PlugIn1,PlugIn2,PlugOut1,PlugOut2; /* in & out plugs */
struct fluid_plug NodeIn1_loss,NodeIn2_loss,NodeOut1_loss,NodeOut2_loss;
struct fluid_plug PlugHexIn1,PlugHexOut1,PlugHexIn2,PlugHexOut2; 

select_unit[unit]++;

/* --------------------------initialize-------------------------- */
if ( (init_flag) && (select_unit[unit]==1) ) { /* allocate memory only once */
 if (mode!=1) { printf("HEAT_EXCH wrong mode unit%d\n",unit);exit (-1); }
 NodeIn1 [heatex_index]=(struct fluid_plug*) malloc(sizeof(struct fluid_plug)); 
 NodeOut1[heatex_index]=(struct fluid_plug*) malloc(sizeof(struct fluid_plug)); 
 NodeIn2 [heatex_index]=(struct fluid_plug*) malloc(sizeof(struct fluid_plug)); 
 NodeOut2[heatex_index]=(struct fluid_plug*) malloc(sizeof(struct fluid_plug)); 

 if ( (deriv_n[unit][1]==0) || (deriv_n[unit][2]==0) || (deriv_n[unit][3]==0) ||
      (deriv_n[unit][4]==0) ) {
  printf("heatexchanger[%d]: no derivatives 1-4 defined ! \n",unit); exit (-1);
 }
 if (C1==-1) 
  Get_C_heatexchanger(unit,n_plate,m_exchanger,V_channel,cp_exchanger,
                      0.5*(deriv_n[unit][1]+deriv_n[unit][3]),cp1 ,&C1);
 else C1*=1e+3; /* use PAR (5), scale to J/kgK */
 if (C2==-1) 
  Get_C_heatexchanger(unit,n_plate,m_exchanger,V_channel,cp_exchanger,
                      0.5*(deriv_n[unit][2]+deriv_n[unit][4]),cp2,&C2);
 else C2*=1e+3; /* use PAR (5), scale to J/kgK */
 if (1) printf ("\n\nC1=%.9f C2=%.9f\n",C1,C2);
 initPlugFluid(NodeIn1[heatex_index],SetQ,/*m*/(C1/(2*cp1)),cp1,
               /*T*/deriv_n[unit][1],/*U*/0,h);
 initPlugFluid(NodeIn2[heatex_index],SetQ,/*m*/(C2/(2*cp2)),cp2,
               /*T*/deriv_n[unit][2],/*U*/0,h);
 initPlugFluid(NodeOut1[heatex_index],SetQ,/*m*/(C1/(2*cp1)),cp1,
               /*T*/deriv_n[unit][3],/*U*/0,h);
 initPlugFluid(NodeOut2[heatex_index],SetQ,/*m*/(C2/(2*cp2)),cp2,
               /*T*/deriv_n[unit][4],/*U*/0,h);


} /* alloc */

if (select_unit[unit]==1) return(1); /* if first_time_flag=1; */
if (select_unit[unit]==3) return(1);  /*return third loop !! */ 

/* init plugs, future: PlugIn1=out_n IN(1); */
initPlugFluid(&PlugIn1,dQin1,mdotIn1*h,cp1,SetT,0,h);
initPlugFluid(&PlugIn2,dQin2,mdotIn2*h,cp2,SetT,0,h);


if (kA==-1) { /* auto calc with Get_kA_heatexchanger() & Get_alpha() */
 mode_calc_kA        = 1; 
 if (init_flag) {
  alpha1=alpha2=8000.0;
  kA=A_plate*(n_plate-2)
            /(1.0/alpha2+s_plate/Lambda_plate+1.0/alpha1);
  Temp1=0.5*(deriv_n[unit][1]+deriv_n[unit][2]);
  Temp2=0.5*(deriv_n[unit][2]+deriv_n[unit][4]);
 }
 else {
  alpha1             = out_o[unit][24]; /* read old values */ 
  alpha2             = out_o[unit][25]; 
  kA                 = out_o[unit][26];
  Temp1              = out_o[unit][31];
  Temp2              = out_o[unit][32];
 }
}
/* no heat transfer if one mdot == 0 */
if ( (mdotIn1==0 ) || (mdotIn2==0) )  noHeatTransferFlag=1;

/* --------------------------------Get_MixedPlug_Fluid In-------- */
//NodeIn1[heatex_index]->Q=PlugIn1.Q; /* static variable */
//NodeIn1_loss.U=NodeIn2_loss.U=2000; /* kA value */
//NodeIn1_loss.T=NodeIn2_loss.T=20; /* Tambient */
//NodeIn1_loss.h=NodeIn2_loss.h=h; /* timestep */

Cdot1=mdotIn1*cp1;
Cdot2=mdotIn2*cp2;

if (!noHeatTransferFlag) ratio=Cdot1/Cdot2;
else ratio=0;
if (mode_calc_kA) Get_kA_heatexchanger(unit,tsim,init_flag,ratio,
                                       b_plate,s_plate,A_plate,
                                       Lambda_plate,delta_channel,
                                       n_plate,a_fit,b_fit,c_fit,
                                       Temp1,     Temp2,
                                       &alpha1,   &alpha2,
                                       cp1,       cp2,
                                       mdotIn1,   mdotIn2,
                                       &kA,
                                       &delta_pressure1, &delta_pressure2);

if (noHeatTransferFlag) kA_loss=/*0.5* */kA;/*stebus!!!!!!!!!!!!!!!!!!*/
else kA_loss=0;
T_amb=NodeOut2[heatex_index]->T;/*stebus!!!!!!!!!!!!!!!!!!*/
initPlugFluid(&NodeIn1_loss,SetQ,0,0,T_amb,kA_loss,h);
T_amb=NodeOut1[heatex_index]->T;/*stebus!!!!!!!!!!!!!!!!!!*/
initPlugFluid(&NodeIn2_loss,SetQ,0,0,T_amb,kA_loss,h);
if (PlugIn1.m==0) PlugIn1.T=deriv_n[unit][1];
if (PlugIn2.m==0) PlugIn2.T=deriv_n[unit][2];

Get_MixedPlug_Fluid(&PlugHexIn1,NodeIn1[heatex_index],&NodeIn1_loss,PlugIn1);
Get_MixedPlug_Fluid(&PlugHexIn2,NodeIn2[heatex_index],&NodeIn2_loss,PlugIn2);

//PlugHexIn1,PlugHexOut1,PlugHexIn2,PlugHexOut2
dQHexIn1  =PlugHexIn1.Q;
mdotHexIn1=mdotIn1; 
dQHexIn2  =PlugHexIn2.Q;
mdotHexIn2=mdotIn2;

if (a) printf("tsim:%.2f Tin1:%.2f T1:%.2f T2:%.2f Tin2:%.2f loss1:%.2f "
  "loss2:%.2f\n" ,tsim,PlugIn1.T,
 NodeIn1[heatex_index]->T,NodeIn2[heatex_index]->T,PlugIn2.T,NodeIn1_loss.Q,
 NodeIn2_loss.Q);   

/* --------------------------------static heatexchanger---------- */

Temp1In=deriv_n[unit][1]=PlugHexIn1.T;
Temp2In=deriv_n[unit][2]=PlugHexIn2.T;

if (!noHeatTransferFlag) { /* there is mass flow, calc Q_transfer ... */
  NTU=kA/Cdot1; /* Number Of Transfer units */
  if (fabs(ratio-1)<=1e-12)/*(ratio==1)*/ {
    cell_ratio=0;
    log_cell_ratio=0;
    efficiency=NTU/(1+NTU);
  }
  else { /*(ratio!=1)*/
    log_cell_ratio=-NTU*(1-ratio); 
    if (log_cell_ratio<100) { 
      cell_ratio=exp (log_cell_ratio);
      efficiency=(1-cell_ratio)/(1-ratio*cell_ratio); 
    }
    else {
      cell_ratio=1e43;
      efficiency=1.0/ratio;
    }
  }
  Q_transfer=h*efficiency*Cdot1*(Temp1In-Temp2In);
}
else { NTU=0;cell_ratio=0;log_cell_ratio=0;efficiency=0;Q_transfer=0; }

if (a) printf("NTU=%.6f ratio=%.6f cell_ratio=%.6f efficiency=%.6f "
              "Q_transfer=%.6f\n",NTU,ratio,cell_ratio,efficiency,Q_transfer);

if ( (efficiency<0) || (efficiency>1) ) { 
  printf("heatexchanger[%d]: efficiency=%.2f Cdot1=%.2f Cdot2=%.2f tsim=%.1f\n",
         unit,efficiency,Cdot1,Cdot2,tsim); exit (-1); 
}

/* energy equation */

PlugHexOut1.Q=dQHexIn1-Q_transfer;
PlugHexOut2.Q=dQHexIn2+Q_transfer;

/* --------------------------------Get_MixedPlug_Fluid Out------- */
initPlugFluid(&PlugHexOut1,PlugHexOut1.Q,PlugHexIn1.m,PlugHexIn1.cp,SetT,0,h);
initPlugFluid(&PlugHexOut2,PlugHexOut2.Q,PlugHexIn2.m,PlugHexIn2.cp,SetT,0,h);
T_amb=NodeIn2[heatex_index]->T;/*stebus!!!!!!!!!!!!!!!!!!*/
initPlugFluid(&NodeOut1_loss,SetQ,0,0,T_amb,kA_loss,h);
T_amb=NodeIn1[heatex_index]->T;/*stebus!!!!!!!!!!!!!!!!!!*/
initPlugFluid(&NodeOut2_loss,SetQ,0,0,T_amb,kA_loss,h);

if (PlugHexOut1.m==0) PlugHexOut1.T=deriv_n[unit][3];
if (PlugHexOut2.m==0) PlugHexOut2.T=deriv_n[unit][4];

if (a) printf ("tsim=%.0f PlugHexOut1.m=%.3f PlugHexOut2.m=%.3f \n",
               tsim,PlugHexOut1.m,PlugHexOut2.m);

if ( !noHeatTransferFlag &&
   ( ( (Temp1In>PlugHexOut2.T +1e-12) && (PlugHexOut1.T<Temp2In -1e-12) ) ||
     ( (Temp1In<PlugHexOut2.T -1e-12) && (PlugHexOut1.T>Temp2In +1e-12) ) ) ) {
 printf ("crossing temperatures in heatexchanger unit[%d]:\n"
         "Temp1In=%.3f PlugHexOut2.T=%.3f PlugHexOut1.T=%.3f Temp2In=%.3f\n",
         unit,Temp1In,PlugHexOut2.T,PlugHexOut1.T,Temp2In);
 exit (-1); 
}


Get_MixedPlug_Fluid(&PlugOut1,
 NodeOut1[heatex_index],&NodeOut1_loss,PlugHexOut1);
Get_MixedPlug_Fluid(&PlugOut2,
 NodeOut2[heatex_index],&NodeOut2_loss,PlugHexOut2);

Temp1Out=deriv_n[unit][2]=NodeOut1[heatex_index]->T;
Temp2Out=deriv_n[unit][4]=NodeOut2[heatex_index]->T;

if (a) printf("Temp1Out=%.3f Temp2Out=%.3f Qp=%.3f\n",
              Temp1Out,Temp2Out,(Q_transfer/h));

    /* TRNSYS:
      TempHotOut=TempHotIn-efficiency*(Cmin/cpHot)*(TempHotIn-TempColdIn);
       dQoutHot=mpinHot*cpHot*h*TempHotOut;
      TempColdOut=efficiency*(Cmin/cpCold)*(TempHotIn-TempColdIn)*TempColdIn;
      dQoutCold=dQinHot+dQinCold-dQoutHot;
      TempColdOut=dQoutCold/(mpinCold*cpCold*h);
      but some problems occured for energy bilance
      we try to loose no energy
    */

if ( (!noHeatTransferFlag) && (fabs(ratio-1)>1e-12)/*(ratio!=1)*/) 
  deltaT_log  =      (Temp1In-Temp2In)*efficiency/NTU;
else /*( (noHeatTransferFlag) || (ratio==1) )*/
  deltaT_log  =       Temp1In-Temp2Out;

deltaT_linear = 0.5*((Temp1In-Temp2Out)+(Temp1Out-Temp2In));

if (deltaT_linear!=0) kA_linear = Q_transfer/(h*deltaT_linear);
else                  kA_linear = 0;

if ( (!noHeatTransferFlag) && (deltaT_log>deltaT_log_max) )
  printf ("warning! in heatexchanger unit[%d] deltaT_log>deltaT_log_max\n"
          "deltaT_log_max=%.3f deltaT_log=%.3f\n",
          unit,deltaT_log_max,deltaT_log);

if (a) printf ("tsim:%.0f\n",tsim);

if (mode_calc_kA) Get_Temp1_Temp2 (unit,tsim,init_flag,ratio,
                                   noHeatTransferFlag,log_cell_ratio,cell_ratio,
                                   b_plate,s_plate,A_plate,
                                   Lambda_plate,delta_channel,
                                   n_plate,deltaT_log,
                                   Temp1Out,Temp2Out,
                                   Temp1In, Temp2In,
                                   alpha1,  alpha2,
                                   kA,
                                   &Temp1,  &Temp2);

if (a) printf ("Temp1In:%.1f Temp1Out:%.1f Temp1:%.1f Temp2In:%.1f "
               "Temp2Out:%.1f Temp2:%.1f \n",Temp1In,Temp1Out,Temp1,
               Temp2In,Temp2Out,Temp2);
		  
if ( (Temp1Out>150) || (Temp2Out>150) || (Temp1Out<-10) || (Temp2Out<-10) ) {
  printf("hexchg:%d Temp1Out:%.1f Temp2Out:%.1f HeatTransferFlag:%d\n"
         "Temp1In:%.1f Temp2In:%.1f \n",
         unit,Temp1Out,Temp2Out,!noHeatTransferFlag,Temp1In,Temp2In);
  exit (-1);
}

/* --------------------------------------set output ------------- */
out_n[unit][1]  = PlugOut1.Q; /*mdotHexIn1*cp1*h*Temp1Out; */
out_n[unit][2]  = mdotIn1; 
out_n[unit][3]  = PlugOut2.Q;
out_n[unit][4]  = mdotIn2;  
out_n[unit][5]  = efficiency;

out_n[unit][6]  = 0; /* sys gain */
out_n[unit][7]  = NodeIn1_loss.Q +NodeIn2_loss.Q  /* sys loss */
                 +NodeOut1_loss.Q+NodeOut2_loss.Q ;
out_n[unit][8]  = NodeIn1 [heatex_index]->Q+NodeIn2 [heatex_index]->Q 
                 +NodeOut1[heatex_index]->Q+NodeOut2[heatex_index]->Q;
out_n[unit][9]  = 0;
//  if ( fabs( (Q_transfer/h)+out_n[unit][9] >1)  ) /*!!!!!!!!!!!!!!!!???????*/
//     printf("Q_transfer:%.1f QWW: %.1f \n",Q_transfer,out_n[unit][9]);

out_n[unit][10] = Temp1Out; 
out_n[unit][11] = Temp2Out;
out_n[unit][13] = Cdot1*(Temp1Out-Temp1In);
out_n[unit][14] = Cdot2*(Temp2Out-Temp2In);
out_n[unit][15] = Q_transfer/h; /* Pexchange */
out_n[unit][16] =(NodeIn1_loss.Q +NodeIn2_loss.Q
                  +NodeOut1_loss.Q+NodeOut2_loss.Q)/h;    /* Ploss, static model */
out_n[unit][17] = deltaT_linear;
out_n[unit][18] = out_o[unit][18]=noHeatTransferFlag;
  /* first_execute_time in out_n[unit][19] */
out_n[unit][20] = Temp1In;
out_n[unit][21] = Temp2In;
out_n[unit][22] = Q_transfer/deltaT_log/h;/* kA_eff */
out_n[unit][23] = deltaT_log;
out_n[unit][24] = alpha1;
out_n[unit][25] = alpha2;
out_n[unit][26] = kA;
out_n[unit][27] = kA/(A_plate*(n_plate-2)) ;/* k */
out_n[unit][28] = deltaT_linear;
out_n[unit][29] = kA_linear;
out_n[unit][30] = kA_linear/(A_plate*(n_plate-2)) ;/* k_linear */; 
out_n[unit][31] = Temp1;
out_n[unit][32] = Temp2;
out_n[unit][33] = delta_pressure1;
out_n[unit][34] = delta_pressure2;

return 1;
}

/* ---------------calculate kA as a function of mp and T----------- */
int Get_kA_heatexchanger(int unit,double tsim,int init_flag,double ratio,
                         double b_plate,double s_plate,double A_plate,
                         double Lambda_plate,double delta_channel,
                         double n_plate,double a_fit,double b_fit,double c_fit,
                         double Temp1,   double Temp2,
                         double *alpha1, double *alpha2,
                         double cp1,     double cp2,
                         double mdotIn1, double mdotIn2,
                         double *kA,
                         double *delta_pressure1, double *delta_pressure2)
{
int a = 0; /* debug all */

if (init_flag&&
    ((b_plate==0)||(s_plate==0)||(A_plate==0)||(Lambda_plate==0)||
    (delta_channel==0)||( n_plate==0)||
    (a_fit==0)||(b_fit==0)||(c_fit==0))) {
  printf ("params for Get_kA_heatexchanger unit[%d] not set, 
          try sort .xfig!\n",unit);
  exit(-1);
}

if (a) printf ("Temp1:%.1f Temp2:%.1f\n",Temp1,Temp2);

Get_alpha (unit,cp1,Temp1,mdotIn1,
           (delta_channel*b_plate*0.5*(n_plate-1))/*A_heattrans*/,
           delta_channel,a_fit,b_fit,c_fit,alpha1,
           (A_plate/b_plate)/*length_channel*/,delta_pressure1);
Get_alpha (unit,cp2,Temp2,mdotIn2,
           (delta_channel*b_plate*0.5*(n_plate-1))/*A_heattrans*/,
           delta_channel,a_fit,b_fit,c_fit,alpha2,
           (A_plate/b_plate)/*length_channel*/,delta_pressure2);
*kA=A_plate*(n_plate-2)
          /(1.0/ *alpha1+s_plate/Lambda_plate+1.0/ *alpha2);
if (a) printf("tsim=%.1f kA=%.2f k=%.6f\n alpha1=%.2f alpha2=%.2f\n",
              tsim,*kA,(*kA/(A_plate*(n_plate-2))),*alpha1,*alpha2);

if (init_flag) printf("delta_channel=%.5f b_plate=%.5f n_plate=%.2f\n"
                      "s_plate=%.5f Lambda_plate=%.2f cp1=%.2f cp2=%.2f\n",
                      delta_channel,b_plate,n_plate,s_plate,Lambda_plate,
                      cp1,cp2);
if (isnan(*kA)==1) {
  printf("heatexchanger unit[%d]: kA=%.5f",unit,*kA);
  exit(-1);
}
return (1);
}/* Get_kA_heatexchanger */

/* --calculate Temp1 & Temp2 for Get_kA_heatexchanger next timestep--- */
int Get_Temp1_Temp2 (int unit,double tsim,int init_flag,double ratio,
                     int noHeatTransferFlag,
                     double log_cell_ratio,double cell_ratio,
                     double b_plate,double s_plate,double A_plate,
                     double Lambda_plate,double delta_channel,
                     double n_plate,double deltaT_log,
                     double Temp1Out,double Temp2Out,
                     double Temp1In, double Temp2In,
                     double alpha1,  double alpha2,
                     double kA,
                     double *Temp1,  double *Temp2)
{
int a = 0; /* debug all */
double Temp1Mid,Temp1Surf,Temp2Mid,Temp2Surf;
double K0_log_factor=1;

if ( (!noHeatTransferFlag) && (fabs(ratio-1)>1e-12)/*(ratio!=1)*/ ) 
  K0_log_factor=2.0*(1.0/(-log_cell_ratio)-1.0/(-1+1/cell_ratio));

/* if noHeatTransferFlag 
K0_log_factor=2.0 *(1.0/(log((Temp1In-Temp2Out)/(Temp1Out-Temp2In))
          -1.0/(-1+ (Temp1In-Temp2Out)/(Temp1Out-Temp2In))));*/

if (a) printf("deltaT_log=%.6f K0_log_factor=%.6f noHeatTransferFlag=%d "
              "kA=%.1f\n",deltaT_log,K0_log_factor,noHeatTransferFlag,kA);
Temp2Mid =Temp2In+0.5*(Temp2Out-Temp2In)*K0_log_factor;
Temp2Surf=Temp2Mid+kA/(alpha2*A_plate*(n_plate-2))*deltaT_log;
*Temp2    =0.5*(Temp2Mid+Temp2Surf);
Temp1Mid  =Temp2Mid+deltaT_log;
Temp1Surf =Temp1Mid -kA/(alpha1*A_plate*(n_plate-2))*deltaT_log;
*Temp1     =0.5*(Temp1Mid+Temp1Surf);
if (a) printf("Temp1Mid =%.6f Temp1Surf =%.6f alpha1 =%.1f\n",
              Temp1Mid,Temp1Surf,alpha1);

if ( (K0_log_factor<-0.00001) || (K0_log_factor>2.00001) ) {
  printf ("heatexchanger unit[%d]: K0_log_factor=%.6f Temp2Mid=%.6f "
          "Temp1Mid=%.6f\n\n",unit,K0_log_factor,Temp2Mid,Temp1Mid);
  exit (-1);
}
return (1);
}/* Get_Temp1_Temp2 */

/* --calculate both sides alpha for Get_kA_heatexchanger----------*/
int Get_alpha (int unit,double cp,double Temp,double mp,double A_heattrans,
               double delta_channel,double a_fit,double b_fit,double c_fit,
               double *alpha,double length_channel,
               double *delta_pressure)

{
double rho,Ny,Lambda,Pr,w,Nu,Re;
if ((a_fit==0)||(b_fit==0)||(c_fit==0)) { 
  printf ("fit params for Get_kA_heatexchanger unit[%d] not set, 
          try sort .xfig!\n",unit);
  exit(-1);
}
rho=Get_rho_fluid (unit,cp,Temp);
Ny=Get_Ny_fluid (unit,cp,Temp);
Lambda=Get_Lambda_fluid (unit,cp,Temp);
Pr=Ny*rho*cp/Lambda;
w=mp/(rho*A_heattrans+1e-16);
Re=w*2*delta_channel/(Ny+1e-16);
/* Nu=0.252*pow(Pr,0.4)*pow(Re,0.7);*/ /* Stupperich FH-Muenster */
/* Nu=0.53*pow(Pr,0.4)*pow(Re,0.63);*/ /* fit for alpha_laval CB76-71E 
                             (Rechenwerte alpha-laval CAS 200 */
/* Nu=0.53*pow(Pr,0.4)*pow(Re,0.72);*/ /* fit for alpha-laval CB14-50 
                             (Messwerte aus ITW Pruefbericht SolvisMax) */
Nu=a_fit*pow(Pr,b_fit)*pow(Re,c_fit);
if (Nu<1) Nu=1; /* just heatconduction */
*alpha=Nu*Lambda/(2*delta_channel);
if (0) printf("Temp=%.6f mp=%.3f rho=%.3f Ny=%.12f Lambda=%.6f \n"
              "Pr=%.6f w=%.6f Re=%.6f Nu=%.6f *alpha=%.6f\n",
              Temp,mp,rho,Ny,Lambda,Pr,w,Re,Nu,*alpha);
#if CALC_PRESSURE_LOSS
{
Get_delta_p (unit,length_channel,delta_channel,rho,w,Re,delta_pressure);
}
#endif
return 0;
}/* Get_alpha */

int Get_delta_p (int unit,double length_channel,double delta_channel,double rho,
                 double w,double Re,double *delta_pressure) 
{
double zeta=length_channel/delta_channel*pow((1.8*log10(Re/7.0)),-2);
*delta_pressure=zeta*rho/2*w*w;
return 0;
}/* Get_delta_p */

/* ---calculate capacity of metal and fluid with init temp-------- */
int Get_C_heatexchanger(int unit,double n_plate,double m_exchanger,
                        double V_channel,double cp_exchanger,double Temp,
                        double cp_fluid,double *C_exchanger)
{
double rho_fluid;
rho_fluid=Get_rho_fluid (unit,cp_fluid,Temp);
*C_exchanger =  0.5 * m_exchanger                         * cp_exchanger
              + 0.5 * (n_plate-1) * V_channel * rho_fluid * cp_fluid;
return 0;
}/* Get_C_heatexchanger */

/* ----------------------end heatexchanger.dynamic.c------------------ */


/* ---thermodynamic data for water and for 40% TYFOCOR L (60% water)------ */
/* water:         fit from Stupperich (FH-Muenster) 
                  after data from VDI-Waermeatlas 1994
   TYFOCOR L 40%: TYFOROP CHEMIE GmbH, Hamburg                             */

double Get_rho_fluid (int unit,double cp_fluid,double T_fluid) /* kg/m^3 */
{
double rho_fluid=-1;
if ( (cp_fluid >4.10*1e+3) && (cp_fluid <4.30*1e+3) )    /* water */
  rho_fluid = 1000.0/(1+pow((fabs((T_fluid-4.0)/588.6)
                       +pow((T_fluid/462),5.6)),1.73));
else {
 if ( (cp_fluid >3.50*1e+3) && (cp_fluid <4.00*1e+3) )   /* 40% TYFOCOR L */
   rho_fluid = 1048.79993536
                 -0.46941906*T_fluid
                 -0.00278992*T_fluid*T_fluid
                 +0.00001411*T_fluid*T_fluid*T_fluid
                 -0.00000003*T_fluid*T_fluid*T_fluid*T_fluid;
 else { printf("wrong cp set in unit[%d]\n",unit); exit(-1); }
}
return (rho_fluid);
}/* Get_rho_fluid */

double Get_Ny_fluid (int unit,double cp_fluid,double T_fluid)
{
double Ny_fluid=-1;
if ( (cp_fluid >4.10*1e+3) && (cp_fluid <4.30*1e+3) )    /* water */
  Ny_fluid = (0.073+pow((0.7625+T_fluid/73.3),-2))*1e-6;
else {
 if ( (cp_fluid >3.50*1e+3) && (cp_fluid <4.00*1e+3) )   /* 40% TYFOCOR L */
   Ny_fluid   = ( 11.84513487  
                  -0.86106660*T_fluid
                  +0.02789554*T_fluid*T_fluid
                  -0.00037135*T_fluid*T_fluid*T_fluid
                  +0.00000168*T_fluid*T_fluid*T_fluid*T_fluid ) *1e-6;
 else { printf("wrong cp set in unit[%d]\n",unit); exit(-1); }
}
return (Ny_fluid);
}/* Get_Ny_fluid */

double Get_Lambda_fluid (int unit,double cp_fluid,double T_fluid)
{
double Lambda_fluid=-1;
if ( (cp_fluid >4.10*1e+3) && (cp_fluid <4.30*1e+3) )    /* water */
  Lambda_fluid = 0.684-pow(fabs(((T_fluid-132)/327.6)),2.3);
else {
 if ( (cp_fluid >3.50*1e+3) && (cp_fluid <4.00*1e+3) )   /* 40% TYFOCOR L */
   Lambda_fluid   = 0.35165993  
                   +0.00177309*T_fluid
                   -0.00000019*T_fluid*T_fluid;
 else { printf("wrong cp set in unit[%d]\n",unit); exit(-1); }
}
return (Lambda_fluid);
}/* Get_Lambda_fluid */
