/* pipe3.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"

/*   pipe3 is a node model according to the following concept:
   The incoming plug displaces the fluid in the pipe up to its own lenght
   in the pipe. From the pipe entrance til this lenght (x_p), all nodes have
   the temperature of the plug. All other nodes receive the fluid displaced
   by the plug. Fluid mixture takes place only within each node, not inbetween
   nodes.
   Therefore, by the number of nodes, the deadtime behaviour can be varied:
   Although the deadtime itself always equals the passage time through the
   pipe, the response curve to a sudden increase in plug temperature can be
   a step-function for an infinite number of nodes or a smooth curve for 
   fewer nodes.   Recommended are about 5-10 nodes.
   Note that although energied and temperatures are only stored for the
   nodes,  the calculation is accurate also for plugs of non-integer lenght
   (in units of nodes). */
 
/* 000516 Euler stabilization for cooling added. 
   000727 comment if pipe-dummy call
*/

/*-------------------1xn PIPE1 -----------------------------PIPE -----------*/

int pipe2(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,qp_sum,
         tsim,init_flag,dummy)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           double **dummy; /* internal out_n for pipe2 calls */
{
  /* following comment is used from script etc/xfig.awk   

    TYPE 31
    PRIORITY  6          used by config routine
    HYDRAULIC 1          1-pole fluid              


    PAR1: 1    [m]      length l of pipe
    PAR2: 10   [mm]     inside diameter di
    PAR3: 4.19 [kJ/kgK] cp fluid (water) 
    PAR4: 0.04 [W/Km]   kA/l
    PAR5: 1.0  [1]      factor for fluid-capacity due to pipe-ambient materials
    PAR6: 5   [1]      n_max nodes 
    PAR7: 1 [1] ceta, hydraulic-field deltaP=deltaP+ceta*Vp^2
  
    INP1: 0 [Ws] deltaQ-inlet
    INP2: 0 [kg/s] mp-inlet
    INP3: 20.0 [C] ambient temperature
    INP4: 0 [Ws] external gain
    
    OUT1: [Ws] dQ-out
    OUT2: [kg/s] mp-out
    OUT3: [C] outlet temperature 
    OUT4: [C] deltaTambient
    OUT5: [] 
    OUT6: [Ws] system gains 
    OUT7: [Ws] system losses
    OUT8: [Ws] system capacity with respect to 0 degrees 
    OUT9: [C] Tmean
    OUT10: [W] P_loss ambient 
    OUT11: [C] T_inlet 
    OUT12: [Ws] dQ_ambient Q11
    OUT13: [C]  Tamb
    OUT14: []  
    OUT15: [] 
    OUT16: [W/K] k_loss_eff

    DERIV1: 10 [C] temp of node 1
 */

#define N_MAX 100 

int a=0; /*-----chriwi's debug flag----------------------------*/
int i=0,j=0;
int n = (int) par[unit][6]; /* 1xn nodes */
int n_p;
int pedantic=1;
double energy [N_MAX+1] ;
double Qi[N_MAX+1][5]; 
double mc,mc_i;
double x_p;
double mp_cp_h;
double dQ_out = 0;
double dQ_in  =out_n[  in_u[unit][1] ]
                    [ in_nr[unit][1] ];
double mp     =out_n[  in_u[unit][2] ]
                    [ in_nr[unit][2] ];
double Tamb   =out_n[ in_u [unit][3] ]
                    [ in_nr[unit][3] ];
double dQgains=out_n[ in_u [unit][4] ]
                    [ in_nr[unit][4] ];
double cp=par[unit][3]*1e+3;
double kA=par[unit][1]*par[unit][4];  /* kA-total */
double Vol=3.14159265*par[unit][2]*par[unit][2]/4*par[unit][1]*1e-3;
double Tmean=0,T_tmp,energy_with_plug,kA_i;
double rho=1,t_dead,T_plug;

if (par[unit][5]>0.8) mc=par[unit][5]*Vol*cp*rho; /* total capacity with  */
else                  mc=             Vol*cp*rho; /* ambient capacites from par 4 */
mp_cp_h=mp*cp*h; /* input-plug */

if (init_flag)  /*----------------- Initialize -----------------------------------*/
  {
  int i;
  if (n<1) { printf("unit:%d auto select par6 (n) \n",unit); 
      par[unit][6]=n=10; }
  if (par[unit][5]<1.0) { printf("unit:%d auto select par5 (factor-fluid-capacity)\n"
                         ,unit); par[unit][5]=1.0; } 

  if (deriv_o[unit][1]<=0) deriv_o[unit][1]=deriv_n[unit][1]=10;
  mc_i=mc/n; 
  for (i=1;i<=n;i++) 
    {
    if (i>1) deriv_o[unit][i]=deriv_n[unit][i]=deriv_o[unit][1]; /* init-temp */
    init_energy[unit][i]=deriv_o[unit][i]*mc_i;
    }
  };  /* end of init-------------------------------------------------------------*/

mc_i=mc/n;       /* node-capacity if n>=1 !! */
kA_i=kA/n;
 
 if (mp_cp_h>0)
 {
  t_dead =mc/mp_cp_h; 
  x_p = (double)(n) / t_dead;  /* endpoint of the incoming plug in the pipe */
  n_p = (int)(x_p);
  T_plug=dQ_in/mp_cp_h;  /* Plug temperature */
 }  
else    
 { t_dead =0;    
  x_p = 0;
  n_p=0;
 }
 
/*-------------------------------------------MAIN-----------------------------*/

for (i=1;i<=n;i++)   /* Loop: Run through nodes of pipe */
{
 if (mp==0)          /* Without incoming flow */
   {
   Qi[i][3]=0; energy_with_plug = deriv_o[unit][i]*mc_i;
   }
 else
   { if (i==1)           /* First node: Only energy from plug enters */
     {
       Qi[i][3] = dQ_in;  /* Incoming energy */
      energy_with_plug=((x_p>=1)?T_plug*mc_i:T_plug*mp_cp_h+ 
                            deriv_o[unit][i]*(mc_i-mp_cp_h));
     }
     else if (i <= n_p)  /* Here energy from plug+neighboring nodes enters */
     { 
       Qi[i][3] = dQ_in/x_p*(x_p-i+1);      /* From plug */
       for (j=i-1;j>0;j--) Qi[i][3] += mc_i * deriv_o[unit][j]; /*neighbors*/ 
       energy_with_plug = T_plug * mc_i;
     }
     else if (i == n_p+1)               /* Same as above, but neighbors also  */
     { Qi[i][3] =  dQ_in/x_p*(x_p-i+1);  /* contribute to energy_with_plug */ 
       for (j=i-1;j>0;j--) Qi[i][3] += mc_i * deriv_o[unit][j];
       energy_with_plug = T_plug *(mp_cp_h-n_p*mc_i) + 
                          deriv_o[unit][1] * ((n_p+1)*mc_i - mp_cp_h);
     }
     else              /* No contribution from plug, only from neighbors */
     { Qi[i][3] = deriv_o[unit][i-n_p-1]*(mp_cp_h-n_p*mc_i);
       for (j=n_p;j>0;j--) Qi[i][3] +=   mc_i * deriv_o[unit][i-j];
       energy_with_plug =  deriv_o[unit][i-n_p-1]*(mp_cp_h - n_p * mc_i) + 
                          deriv_o[unit][i-n_p  ]*((n_p+1)*mc_i-mp_cp_h);
     }
   }    
   T_tmp =  energy_with_plug/mc_i;
   /*Qi[i][1]=mc_i*(T_tmp-Tamb)*(1.0 - exp(-kA_i *h/mc_i)); exp.cooling */ 
   Qi[i][1]=(T_tmp-Tamb)*kA_i*h;  /* Linear cooling */
   /* check time constant due to kA_i */
   T_tmp=(energy_with_plug-Qi[i][1])/mc_i; /* New temp. with amb losses */
   if (   ( (Qi[i][1]>0) && (T_tmp<Tamb) ) /* instable */
      || ( (Qi[i][1]<0) && (T_tmp>Tamb) ) 
     )
   {
   if (pedantic) printf("pipe1[%d]:stabilization T_tmp:%.1f Tamb:%.1f Qi:%.1f \n",
          unit,T_tmp,Tamb,Qi[i][1]);
   Qi[i][1]=energy_with_plug-Tamb*mc_i;/* set Tnode to Tamb */
   } 
   Qi[i][2]=dQgains/n;                 /* external gains */
   T_tmp= (energy_with_plug - Qi[i][1] + Qi[i][2]) /mc_i;
   qp_sum[unit][i][1]+=Qi[i][1];
   qp_sum[unit][i][2]+=Qi[i][2];
   qp_sum[unit][i][3]+=Qi[i][3];
   
   if (a) 
   { T_tmp= (energy_with_plug - Qi[i][1] + Qi[i][2]) /mc_i;
      printf("pipe2[%d],node:%d,T_old:%.2f,T_new:%.2f,x_p:%f,T_plug%.2f\n",
             unit,i ,deriv_o[unit][i],T_tmp,x_p,T_plug);
   }
}
 /* Calculate energy leaving the pipe */
if (x_p < n)       /* if plug shorter than pipe */
{ dQ_out = deriv_o[unit][n-n_p]*(mp_cp_h-n_p*mc_i);   /* dQout last element */
  for (j=0;j<n_p;j++) dQ_out += deriv_o[unit][n-j]*mc_i;
}
  else             /* if plug longer than pipe */ 
{ dQ_out = T_plug * (mp_cp_h - n*mc_i);  
  for (j=1;j<=n;j++) dQ_out += deriv_o[unit][j]*mc_i;
}
Qi[n+1][3]= dQ_out;
qp_sum[unit][n+1][3]+=Qi[n+1][3];

for (i=1;i<=n;i+=1) 
  {
  energy[i]=init_energy[unit][i]
                -qp_sum[unit][i  ][1]       /* Q_loss to ambient */
                +qp_sum[unit][i  ][2]       /* Q_gain */
                +qp_sum[unit][i  ][3]       /* Q_in fluid */
                -qp_sum[unit][i+1][3];      /* Q_out fluid */
  deriv_n[unit][i]=energy[i]/mc_i;
  Tmean+=deriv_n[unit][i]/n;

  if (a) printf("Unit:%d,Knoten:%d,T_new:%.2f\n",unit,i,deriv_n[unit][i]);
  /*  if (a) printf("deriv_n[unit][%d]:%.2f\n",i,deriv_n[unit][i]); */


  }


/* -------WARNINGS----------------------------------------------------------------*/
if ( (pedantic) && ( fabs(t_dead)>100.0*(double) (n)/h) ) {
 printf("pipe1:%d deadtime=%.1f [s]>100*n/h=%.1f [s] !!\n",unit
       ,t_dead,100.0*(double) (n)/h);
 }
if ( fabs (deriv_n[unit][n])>300) 
        { printf("pipe gau unit:%d init_energy[%d]:%.2f,T[%d]:%.1f \n"
                 ,unit,n,init_energy[unit][n],n,deriv_n[unit][n]);  exit (-1); }
/*-----------------------------------------END of-MAIN-----------------------------*/

/*-------------- set outputs -----------------------------*/
if (dummy!=NULL)
 {
 if (a) printf("internal call of pipe2: unit:%d\n",unit);
 out_n=dummy;
 }

out_n[unit][1]=dQ_out;  /* dQ_out */
out_n[unit][2]=mp;
if (mp!=0) out_n[unit][3]=dQ_out/mp_cp_h;  /* mean outlet temperature */
else  out_n[unit][3]=deriv_n[unit][n];

out_n[unit][4]=Tmean-Tamb; /*  deltaTambient */
out_n[unit][5]=(dQ_out-dQ_in)/h; /* P_loss [W] */

/*System Energy Balance--------------------------------------------- */
out_n[unit][6]=0; for (i=1;i<=n;i++) out_n[unit][6]+=Qi[i][2]; /* gains  [Ws] */
out_n[unit][7]=0; for (i=1;i<=n;i++) out_n[unit][7]+=Qi[i][1]; /* system losses */
out_n[unit][8]=0; for (i=1;i<=n;i++) out_n[unit][8]+=energy[i];

out_n[unit][9]=Tmean;
out_n[unit][10]=out_n[unit][7]/h;
if (mp_cp_h>0) out_n[unit][11] = dQ_in/mp_cp_h;         /* Tin */
else           out_n[unit][11] = deriv_n[unit][1];

out_n[unit][12] = out_n[unit][7] ; /* dQ_ambient Q11 */
out_n[unit][13] = Tamb;

out_n[unit][16] = kA;

/*if ((mp!=0)&&(unit==23)) printf("%lf  %lf %lf %lf  %lf %lf %lf %d\n",tsim,T_plug,out_n[unit][3],Tmean,x_p,t_dead*h,mp,n);*/

return (1);
}
/*-------------------------------------------------------------------*/
















